@@ -120,7 +120,7 @@ namespace Nereid
         {
             primary_vbox = new VBox ();
             
-            Widget menu = ActionService.UIManager.GetWidget ("/MainMenu");
+            Widget menu = new MainMenu ();
             menu.Show ();
             primary_vbox.PackStart (menu, false, false, 0);
            
@@ -147,9 +147,7 @@ namespace Nereid
             
             primary_vbox.PackStart (toolbar_alignment, false, false, 0);
             
-            Widget button = ActionService.PlaybackActions["NextAction"].CreateToolItem ();
-            Menu menu = ActionService.PlaybackActions.ShuffleActions.CreateMenu ();
-            MenuButton next_button = new MenuButton (button, menu, true);
+            Widget next_button = new NextButton (ActionService);
             next_button.Show ();
             ActionService.PopulateToolbarPlaceholder (header_toolbar, "/HeaderToolbar/NextArrowButton", next_button);
             
@@ -189,7 +187,7 @@ namespace Nereid
             source_view.SetSizeRequest (125, -1);
             view_container.SetSizeRequest (425, -1);
             
-            views_pane.Pack1 (source_box, true, false);
+            views_pane.Pack1 (source_box, false, false);
             views_pane.Pack2 (view_container, true, false);
             
             source_box.ShowAll ();
@@ -205,10 +203,19 @@ namespace Nereid
             footer_toolbar.ShowArrow = false;
             footer_toolbar.ToolbarStyle = ToolbarStyle.BothHoriz;
 
+            EventBox status_event_box = new EventBox ();
+            status_event_box.ButtonPressEvent += delegate (object o, ButtonPressEventArgs args) {
+                Source source = ServiceManager.SourceManager.ActiveSource;
+                if (source != null) {
+                    source.CycleStatusFormat ();
+                    UpdateSourceInformation ();
+                }
+            };
             status_label = new Label ();
+            status_event_box.Add (status_label);
             
             Alignment status_align = new Alignment (0.5f, 0.5f, 1.0f, 1.0f);
-            status_align.Add (status_label);
+            status_align.Add (status_event_box);
 
             RepeatActionButton repeat_button = new RepeatActionButton ();
             repeat_button.SizeAllocated += delegate (object o, Gtk.SizeAllocatedArgs args) {
@@ -231,7 +238,10 @@ namespace Nereid
             // Service events
             ServiceManager.SourceManager.ActiveSourceChanged += OnActiveSourceChanged;
             ServiceManager.SourceManager.SourceUpdated += OnSourceUpdated;
-            ServiceManager.PlayerEngine.EventChanged += OnPlayerEngineEventChanged;
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, 
+                PlayerEvent.StartOfStream |
+                PlayerEvent.TrackInfoUpdated |
+                PlayerEvent.EndOfStream);
             
             ActionService.TrackActions ["SearchForSameArtistAction"].Activated += OnProgrammaticSearch;
             ActionService.TrackActions ["SearchForSameAlbumAction"].Activated += OnProgrammaticSearch;
@@ -243,14 +253,23 @@ namespace Nereid
             };
             
             composite_view.TrackView.RowActivated += delegate (object o, RowActivatedArgs<TrackInfo> args) {
-                SetPlaybackControllerSource (ServiceManager.SourceManager.ActiveSource);
-                ServiceManager.PlayerEngine.OpenPlay (args.RowValue);
+                Source source = ServiceManager.SourceManager.ActiveSource;
+                if (source is ITrackModelSource) {
+                    ServiceManager.PlaybackController.Source = (ITrackModelSource)source;
+                    ServiceManager.PlayerEngine.OpenPlay (args.RowValue);
+                }
             };
 
             source_view.RowActivated += delegate {
-                SetPlaybackControllerSource (ServiceManager.SourceManager.ActiveSource);
-                if (GtkUtilities.NoImportantModifiersAreSet (Gdk.ModifierType.ControlMask)) {
-                    ServiceManager.PlaybackController.Next ();
+                Source source = ServiceManager.SourceManager.ActiveSource;
+                if (source is ITrackModelSource) {
+                    ServiceManager.PlaybackController.NextSource = (ITrackModelSource)source;
+                    // Allow changing the play source without stopping the current song by
+                    // holding ctrl when activating a source. After the song is done, playback will
+                    // continue from the new source.
+                    if (GtkUtilities.NoImportantModifiersAreSet (Gdk.ModifierType.ControlMask)) {
+                        ServiceManager.PlaybackController.Next ();
+                    }
                 }
             };
             
@@ -333,28 +352,24 @@ namespace Nereid
 
             view_container.Header.Visible = source.Properties.Contains ("Nereid.SourceContents.HeaderVisible") ?
                 source.Properties.Get<bool> ("Nereid.SourceContents.HeaderVisible") : true;
+
+            view_container.DiskUsageVisible = (source is IDiskUsageReporter);
             
-            UpdateStatusBar ();
+            UpdateSourceInformation ();
             view_container.SearchEntry.Ready = true;
         }
 
         private void OnSourceUpdated (SourceEventArgs args)
         {
             if (args.Source == ServiceManager.SourceManager.ActiveSource) {
-                UpdateStatusBar ();
+                UpdateSourceInformation ();
                 view_container.Title = args.Source.Name;
             }
         }
         
-        private void OnPlayerEngineEventChanged (object o, PlayerEngineEventArgs args) 
+        private void OnPlayerEvent (PlayerEventArgs args) 
         {
-            switch (args.Event) {
-                case PlayerEngineEvent.StartOfStream:
-                case PlayerEngineEvent.TrackInfoUpdated:
-                case PlayerEngineEvent.EndOfStream:
-                    UpdateTitle ();
-                    break;
-            }
+            UpdateTitle ();
         }
         
 #endregion
@@ -470,28 +485,34 @@ namespace Nereid
 
         private void HandleTrackModelReloaded (object sender, EventArgs args)
         {
-            UpdateStatusBar ();
+            UpdateSourceInformation ();
         }
 
-        private void SetPlaybackControllerSource (Source source)
+        private void UpdateSourceInformation ()
         {
-            // Set the source from which to play to the current source since
-            // the user manually began playback from this source
-            if (!(source is ITrackModelSource)) {
-                source = ServiceManager.SourceManager.DefaultSource;
-            }
-            
-            ServiceManager.PlaybackController.Source = (ITrackModelSource)source;    
-        }
-
-        private void UpdateStatusBar ()
-        {
-            if (ServiceManager.SourceManager.ActiveSource == null) {
+            Source source = ServiceManager.SourceManager.ActiveSource;
+            if (source == null) {
                 status_label.Text = String.Empty;
                 return;
             }
 
-            status_label.Text = ServiceManager.SourceManager.ActiveSource.GetStatusText ();
+            if (source is IDiskUsageReporter) {
+                IDiskUsageReporter reporter = source as IDiskUsageReporter;
+
+                double fraction = (double)reporter.BytesUsed / (double)reporter.BytesCapacity;
+
+                view_container.DiskUsageBar.Fraction = fraction;
+                view_container.DiskUsageBar.Text = String.Format(
+                    Catalog.GetString("{0} of {1}"),
+                    new Hyena.Query.FileSizeQueryValue (reporter.BytesUsed).ToUserQuery (),
+                    new Hyena.Query.FileSizeQueryValue (reporter.BytesCapacity).ToUserQuery ()
+                );
+
+                //string tooltip = dapSource.DiskUsageString + " (" + dapSource.DiskAvailableString + ")";
+                //toolTips.SetTip (dapDiskUsageBar, tooltip, tooltip);
+            }
+
+            status_label.Text = source.GetStatusText ();
         }
 
 #endregion
