@@ -45,7 +45,9 @@ using Banshee.Equalizer;
 
 namespace Banshee.MediaEngine
 {
-    public class PlayerEngineService : IService, IPlayerEngineService
+    public delegate bool TrackInterceptHandler (TrackInfo track);
+
+    public class PlayerEngineService : IInitializeService, IRequiredService, IPlayerEngineService, IDisposable
     {   
         private List<PlayerEngine> engines = new List<PlayerEngine> ();
         private PlayerEngine active_engine;
@@ -54,23 +56,26 @@ namespace Banshee.MediaEngine
 
         private string preferred_engine_id = null;
 
-        public event PlayerEngineEventHandler EventChanged;
-        public event PlayerEngineStateHandler StateChanged;
         public event EventHandler PlayWhenIdleRequest;
+        public event TrackInterceptHandler TrackIntercept;
         
-        private event DBusPlayerEngineEventHandler dbus_event_changed;
-        event DBusPlayerEngineEventHandler IPlayerEngineService.EventChanged {
+        private event DBusPlayerEventHandler dbus_event_changed;
+        event DBusPlayerEventHandler IPlayerEngineService.EventChanged {
             add { dbus_event_changed += value; }
             remove { dbus_event_changed -= value; }
         }
 
-        private event DBusPlayerEngineStateHandler dbus_state_changed;
-        event DBusPlayerEngineStateHandler IPlayerEngineService.StateChanged {
+        private event DBusPlayerStateHandler dbus_state_changed;
+        event DBusPlayerStateHandler IPlayerEngineService.StateChanged {
             add { dbus_state_changed += value; }
             remove { dbus_state_changed -= value; }
         }
         
         public PlayerEngineService ()
+        {
+        }
+        
+        public void Initialize ()
         {
             preferred_engine_id = EngineSchema.Get();
             
@@ -78,7 +83,7 @@ namespace Banshee.MediaEngine
                 default_engine = engines[0];
             }
             
-            foreach (TypeExtensionNode node in AddinManager.GetExtensionNodes ("/Banshee/PlayerEngines/PlayerEngine")) {
+            foreach (TypeExtensionNode node in AddinManager.GetExtensionNodes ("/Banshee/MediaEngine/PlayerEngine")) {
                 LoadEngine (node);
             }
             
@@ -109,10 +114,9 @@ namespace Banshee.MediaEngine
         
         private void LoadEngine (PlayerEngine engine)
         {
-            engine.StateChanged += OnEngineStateChanged;
             engine.EventChanged += OnEngineEventChanged;
 
-            if(engine.Id == preferred_engine_id) {
+            if (engine.Id == preferred_engine_id) {
                 DefaultEngine = engine;
             } else {
                 if (active_engine == null) {
@@ -124,31 +128,38 @@ namespace Banshee.MediaEngine
 
         public void Dispose ()
         {
+            MetadataService.Instance.HaveResult -= OnMetadataServiceHaveResult;
+            
+            foreach (PlayerEngine engine in engines) {
+                engine.Dispose ();
+            }
+            
+            active_engine = null;
+            default_engine = null;
+            pending_engine = null;
+            
+            preferred_engine_id = null;
+            
+            engines.Clear ();
         }
         
         private void OnMetadataServiceHaveResult (object o, MetadataLookupResultArgs args)
         {
             if (CurrentTrack != null && args.Track == CurrentTrack) {
                 foreach (StreamTag tag in args.ResultTags) {
-                    StreamTagger.TrackInfoMerge(CurrentTrack, tag);
+                    StreamTagger.TrackInfoMerge (CurrentTrack, tag);
                 }
                 
-                PlayerEngineEventArgs eventargs = new PlayerEngineEventArgs ();
-                eventargs.Event = PlayerEngineEvent.TrackInfoUpdated;
-                OnEngineEventChanged (active_engine, eventargs);
+                OnEngineEventChanged (new PlayerEventArgs (PlayerEvent.TrackInfoUpdated));
             }
         }
         
-        private void OnEngineStateChanged (object o, PlayerEngineStateArgs args)
+        private void HandleStateChange (PlayerEventStateChangeArgs args)
         {
-            if (o != active_engine) {
-                return;
-            }
-            
-            if (args.State == PlayerEngineState.Loaded && CurrentTrack != null) {
+            if (args.Current == PlayerState.Loaded && CurrentTrack != null) {
                 active_engine.Volume = (ushort) VolumeSchema.Get ();
                 MetadataService.Instance.Lookup (CurrentTrack);
-            } else if (args.State == PlayerEngineState.Ready) {
+            } else if (args.Current == PlayerState.Ready) {
                 // Enable our preferred equalizer if it exists and was enabled last time.
                 if (SupportsEqualizer && EqualizerSetting.EnabledSchema.Get ()) {
                     string name = EqualizerSetting.PresetSchema.Get();
@@ -172,48 +183,39 @@ namespace Banshee.MediaEngine
                 }
             }
             
-            PlayerEngineStateHandler handler = StateChanged;
-            if (handler != null) {
-                handler (o, args);
-            }
-            
-            DBusPlayerEngineStateHandler dbus_handler = dbus_state_changed;
+            DBusPlayerStateHandler dbus_handler = dbus_state_changed;
             if (dbus_handler != null) {
-                dbus_handler (args.State.ToString ().ToLower ());
+                dbus_handler (args.Current.ToString ().ToLower ());
             }
         }
 
-        private void OnEngineEventChanged (object o, PlayerEngineEventArgs args)
+        private void OnEngineEventChanged (PlayerEventArgs args)
         {
-            if (o != active_engine) {
-                return;
-            }
-            
             if (CurrentTrack != null) {
-                if (args.Event == PlayerEngineEvent.Error 
+                if (args.Event == PlayerEvent.Error 
                     && CurrentTrack.PlaybackError == StreamPlaybackError.None) {
                     CurrentTrack.PlaybackError = StreamPlaybackError.Unknown;
-                } else if (args.Event == PlayerEngineEvent.Iterate 
+                } else if (args.Event == PlayerEvent.Iterate 
                     && CurrentTrack.PlaybackError != StreamPlaybackError.None) {
                     CurrentTrack.PlaybackError = StreamPlaybackError.None;
                 }
             }
             
-            PlayerEngineEventHandler handler = EventChanged;
-            if (handler != null) {
-                handler(o, args);
-            }
+            RaiseEvent (args);
             
             // Do not raise iterate across DBus to avoid so many calls;
             // DBus clients should do their own iterating and 
             // event/state checking locally
-            if (args.Event == PlayerEngineEvent.Iterate) {
+            if (args.Event == PlayerEvent.Iterate) {
                 return;
             }
             
-            DBusPlayerEngineEventHandler dbus_handler = dbus_event_changed;
+            DBusPlayerEventHandler dbus_handler = dbus_event_changed;
             if (dbus_handler != null) {
-                dbus_handler (args.Event.ToString ().ToLower (), args.Message ?? String.Empty, args.BufferingPercent);
+                dbus_handler (args.Event.ToString ().ToLower (), 
+                    args is PlayerEventErrorArgs ? ((PlayerEventErrorArgs)args).Message : String.Empty, 
+                    args is PlayerEventBufferingArgs ? ((PlayerEventBufferingArgs)args).Progress : 0
+                );
             }
         }
         
@@ -225,13 +227,25 @@ namespace Banshee.MediaEngine
             }
         }
         
+        private bool OnTrackIntercept (TrackInfo track)
+        {
+            TrackInterceptHandler handler = TrackIntercept;
+            if (handler == null) {
+                return false;
+            }
+            
+            bool handled = false;
+            
+            foreach (TrackInterceptHandler single_handler in handler.GetInvocationList ()) {
+                handled |= single_handler (track);
+            }
+            
+            return handled;
+        }
+        
         public void Open (TrackInfo track)
         {
-            if (!track.CanPlay) {
-                return;
-            }
-               
-            OpenCheck (track);
+            OpenPlay (track, false);
         }
         
         public void Open (SafeUri uri)
@@ -246,7 +260,12 @@ namespace Banshee.MediaEngine
         
         public void OpenPlay (TrackInfo track)
         {
-            if (!track.CanPlay) {
+            OpenPlay (track, true);
+        }
+        
+        private void OpenPlay (TrackInfo track, bool play)
+        {
+            if (!track.CanPlay || OnTrackIntercept (track)) {
                 return;
             }
         
@@ -254,7 +273,8 @@ namespace Banshee.MediaEngine
                 OpenCheck (track);
                 active_engine.Play ();
             } catch (Exception e) {
-                Log.Error (Catalog.GetString ("Problem with Player Engine"), e.Message);
+                Log.Exception (e);
+                Log.Error (Catalog.GetString ("Problem with Player Engine"), e.Message, true);
                 Close ();
                 ActiveEngine = default_engine;
             }
@@ -262,7 +282,7 @@ namespace Banshee.MediaEngine
         
         private void OpenCheck (object o)
         {
-            if (CurrentState == PlayerEngineState.NotReady) {
+            if (CurrentState == PlayerState.NotReady) {
                 throw new InvalidOperationException (String.Format ("Player engine {0} is in the NotReady state", 
                     active_engine.GetType ().FullName));
             }
@@ -278,14 +298,33 @@ namespace Banshee.MediaEngine
             } else {
                 return;
             }
+
+            IncrementLastPlayed ();
             
             FindSupportingEngine (uri);
             CheckPending ();
             
             if (track != null) {
                 active_engine.Open (track);
+                incremented_last_played = false;
             } else if (uri != null) {
                 active_engine.Open (uri);
+                incremented_last_played = false;
+            }
+        }
+
+        private bool incremented_last_played = true;
+        public void IncrementLastPlayed ()
+        {
+            if (!incremented_last_played && active_engine.CurrentTrack != null) {
+                // If we're at least 50% done playing a song, mark it as played, otherwise as skipped
+                // If the Length <= 0, assume the song was finished and increment its play count
+                if (active_engine.Length <= 0 || active_engine.Position >= active_engine.Length / 2) {
+                    active_engine.CurrentTrack.IncrementPlayCount ();
+                } else {
+                    active_engine.CurrentTrack.IncrementSkipCount ();
+                }
+                incremented_last_played = true;
             }
         }
         
@@ -321,8 +360,14 @@ namespace Banshee.MediaEngine
         
         public void Close ()
         {
+            Close (false);
+        }
+        
+        public void Close (bool fullShutdown)
+        {
+            IncrementLastPlayed ();
             active_engine.Reset ();
-            active_engine.Close ();
+            active_engine.Close (fullShutdown);
         }
         
         public void Play ()
@@ -342,10 +387,10 @@ namespace Banshee.MediaEngine
         public void TogglePlaying ()
         {
             switch (CurrentState) {
-                case PlayerEngineState.Idle:
+                case PlayerState.Idle:
                     OnPlayWhenIdleRequest ();
                     break;
-                case PlayerEngineState.Playing:
+                case PlayerState.Playing:
                     Pause ();
                     break;
                 default:
@@ -354,6 +399,15 @@ namespace Banshee.MediaEngine
             }
         }
         
+        public void VideoExpose (IntPtr window, bool direct)
+        {
+            active_engine.VideoExpose (window, direct);
+        }
+        
+        public IntPtr VideoWindow {
+            set { active_engine.VideoWindow = value; }
+        }
+        
         public void TrackInfoUpdated ()
         {
             active_engine.TrackInfoUpdated ();
@@ -361,14 +415,20 @@ namespace Banshee.MediaEngine
         
         public bool IsPlaying (TrackInfo track)
         {
-            return CurrentState != PlayerEngineState.Idle && CurrentState != PlayerEngineState.NotReady && 
-                track != null && track.TrackEqual (CurrentTrack);
+            return IsPlaying () && track != null && track.TrackEqual (CurrentTrack);
+        }
+        
+        public bool IsPlaying ()
+        {
+            return CurrentState == PlayerState.Playing || 
+                CurrentState == PlayerState.Paused || 
+                CurrentState == PlayerState.Loaded;
         }
 
         private void CheckPending ()
         {
             if(pending_engine != null && pending_engine != active_engine) {
-                if(active_engine.CurrentState == PlayerEngineState.Idle) {
+                if(active_engine.CurrentState == PlayerState.Idle) {
                     Close ();
                 }
                 
@@ -393,7 +453,7 @@ namespace Banshee.MediaEngine
             get { return CurrentSafeUri == null ? String.Empty : CurrentSafeUri.AbsoluteUri; }
         }
         
-        public PlayerEngineState CurrentState {
+        public PlayerState CurrentState {
             get { return active_engine.CurrentState; }
         }
         
@@ -401,7 +461,7 @@ namespace Banshee.MediaEngine
             get { return CurrentState.ToString ().ToLower (); }
         }
         
-        public PlayerEngineState LastState {
+        public PlayerState LastState {
             get { return active_engine.LastState; }
         }
         
@@ -435,6 +495,10 @@ namespace Banshee.MediaEngine
             get { return ((active_engine is IEqualizer) && active_engine.SupportsEqualizer); }
         }
         
+        public bool SupportsVideo {
+            get { return active_engine.SupportsVideo; }
+        }
+        
         public uint Length {
             get { 
                 uint length = active_engine.Length;
@@ -471,6 +535,128 @@ namespace Banshee.MediaEngine
             get { return engines; }
         }
         
+#region Player Event System
+
+        private LinkedList<PlayerEventHandlerSlot> event_handlers = new LinkedList<PlayerEventHandlerSlot> ();
+        
+        private struct PlayerEventHandlerSlot
+        {
+            public PlayerEvent EventMask;
+            public PlayerEventHandler Handler;
+            
+            public PlayerEventHandlerSlot (PlayerEvent mask, PlayerEventHandler handler)
+            {
+                EventMask = mask;
+                Handler = handler;
+            }
+        }
+        
+        private const PlayerEvent event_all_mask = PlayerEvent.Iterate
+            | PlayerEvent.StateChange
+            | PlayerEvent.StartOfStream
+            | PlayerEvent.EndOfStream
+            | PlayerEvent.Buffering
+            | PlayerEvent.Seek
+            | PlayerEvent.Error
+            | PlayerEvent.Volume
+            | PlayerEvent.Metadata
+            | PlayerEvent.TrackInfoUpdated;
+        
+        private const PlayerEvent event_default_mask = event_all_mask & ~PlayerEvent.Iterate;
+        
+        private static void VerifyEventMask (PlayerEvent eventMask)
+        {
+            if (eventMask <= PlayerEvent.None || eventMask > event_all_mask) {
+                throw new ArgumentOutOfRangeException ("eventMask", "A valid event mask must be provided");
+            }
+        }
+        
+        public void ConnectEvent (PlayerEventHandler handler)
+        {
+            ConnectEvent (handler, event_default_mask, false);
+        }
+        
+        public void ConnectEvent (PlayerEventHandler handler, PlayerEvent eventMask)
+        {
+            ConnectEvent (handler, eventMask, false);
+        }
+        
+        public void ConnectEvent (PlayerEventHandler handler, bool connectAfter)
+        {
+            ConnectEvent (handler, event_default_mask, connectAfter);
+        }
+        
+        public void ConnectEvent (PlayerEventHandler handler, PlayerEvent eventMask, bool connectAfter)
+        {
+            lock (event_handlers) {
+                VerifyEventMask (eventMask);
+            
+                PlayerEventHandlerSlot slot = new PlayerEventHandlerSlot (eventMask, handler);
+                
+                if (connectAfter) {
+                    event_handlers.AddLast (slot);
+                } else {
+                    event_handlers.AddFirst (slot);
+                }
+            }
+        }
+        
+        private LinkedListNode<PlayerEventHandlerSlot> FindEventNode (PlayerEventHandler handler)
+        {
+            LinkedListNode<PlayerEventHandlerSlot> node = event_handlers.First;
+            while (node != null) {
+                if (node.Value.Handler == handler) {
+                    return node;
+                }
+                node = node.Next;
+            }
+            
+            return null;
+        }
+        
+        public void DisconnectEvent (PlayerEventHandler handler)
+        {
+            lock (event_handlers) {
+                LinkedListNode<PlayerEventHandlerSlot> node = FindEventNode (handler);
+                if (node != null) {
+                    event_handlers.Remove (node);
+                }
+            }
+        }
+        
+        public void ModifyEvent (PlayerEvent eventMask, PlayerEventHandler handler)
+        {
+            lock (event_handlers) {
+                VerifyEventMask (eventMask);
+                
+                LinkedListNode<PlayerEventHandlerSlot> node = FindEventNode (handler);
+                if (node != null) {
+                    PlayerEventHandlerSlot slot = node.Value;
+                    slot.EventMask = eventMask;
+                    node.Value = slot;
+                }
+            }
+        }
+        
+        private void RaiseEvent (PlayerEventArgs args)
+        {
+            lock (event_handlers) {
+                if (args.Event == PlayerEvent.StateChange && args is PlayerEventStateChangeArgs) {
+                    HandleStateChange ((PlayerEventStateChangeArgs)args);
+                }
+            
+                LinkedListNode<PlayerEventHandlerSlot> node = event_handlers.First;
+                while (node != null) {
+                    if ((node.Value.EventMask & args.Event) == args.Event) {
+                        node.Value.Handler (args);
+                    }
+                    node = node.Next;
+                }
+            }
+        }
+
+#endregion
+        
         string IService.ServiceName {
             get { return "PlayerEngine"; }
         }
