@@ -53,9 +53,9 @@ namespace Banshee.Gui
                     Catalog.GetString ("Import _Media..."), "<control>I",
                     Catalog.GetString ("Import media from a variety of sources"), OnImport),
 
-                new ActionEntry ("ImportPlaylistAction", null,
+                /*new ActionEntry ("ImportPlaylistAction", null,
                     Catalog.GetString ("Import Playlist..."), null,
-                    Catalog.GetString ("Import a playlist"), OnImportPlaylist),
+                    Catalog.GetString ("Import a playlist"), OnImportPlaylist),*/
 
                 new ActionEntry ("OpenLocationAction", null, 
                     Catalog.GetString ("Open _Location..."), "<control>L",
@@ -154,18 +154,18 @@ namespace Banshee.Gui
                 radio_track.ParsingPlaylistEvent += delegate {
                     if (radio_track.PlaybackError != StreamPlaybackError.None) {
                         Log.Error (Catalog.GetString ("Error opening stream"), 
-                            Catalog.GetString ("Could not open stream or playlist"));
+                            Catalog.GetString ("Could not open stream or playlist"), true);
                         radio_track = null;
                     }
                 };
                 radio_track.Play ();
             } catch {
                 Log.Error (Catalog.GetString ("Error opening stream"), 
-                    Catalog.GetString("Problem parsing playlist"));
+                    Catalog.GetString("Problem parsing playlist"), true);
             }
         }
 
-        private void OnImportPlaylist (object o, EventArgs args)
+        /*private void OnImportPlaylist (object o, EventArgs args)
         {
             // Prompt user for location of the playlist.
             Banshee.Gui.Dialogs.FileChooserDialog chooser = new Banshee.Gui.Dialogs.FileChooserDialog(
@@ -226,7 +226,7 @@ namespace Banshee.Gui
                 md.Destroy();
                 return;
             }
-        }
+        }*/
         
         private void OnQuit (object o, EventArgs args)
         {
@@ -239,9 +239,12 @@ namespace Banshee.Gui
 
         private void OnPreferences (object o, EventArgs args)
         {
-            Dialogs.PreferencesDialog dialog = new Dialogs.PreferencesDialog ();
-            dialog.Run ();
-            dialog.Destroy ();
+            try {
+                Banshee.Preferences.Gui.PreferenceDialog dialog = new Banshee.Preferences.Gui.PreferenceDialog ();
+                dialog.Run ();
+                dialog.Destroy ();
+            } catch (ApplicationException) {
+            }
         }
 
         private void OnExtensions (object o, EventArgs args)
