@@ -92,8 +92,8 @@ namespace Banshee.Gui
                     Catalog.GetString("Unselect all tracks"), OnSelectNone),
 
                 new ActionEntry ("TrackPropertiesAction", Stock.Edit,
-                    Catalog.GetString ("_Edit Track Metadata"), "E",
-                    Catalog.GetString ("Edit metadata on selected tracks"), OnTrackProperties),
+                    Catalog.GetString ("_Edit Track Information"), "E",
+                    Catalog.GetString ("Edit information on selected tracks"), OnTrackProperties),
 
                 new ActionEntry ("AddToPlaylistAction", null,
                     Catalog.GetString ("Add _to Playlist"), null,
@@ -197,6 +197,10 @@ namespace Banshee.Gui
 
         private void UpdateActions ()
         {
+            if (TrackSelector == null || TrackSelector.TrackSelectionProxy == null) {
+                return;
+            }
+            
             Hyena.Collections.Selection selection = TrackSelector.TrackSelectionProxy.Selection;
             Source source = ServiceManager.SourceManager.ActiveSource;
             bool in_database = source is DatabaseSource;
@@ -218,14 +222,8 @@ namespace Banshee.Gui
                        "SearchMenuAction", "SearchForSameArtistAction", "SearchForSameAlbumAction"
                     );
 
-                    UpdateAction ("RemoveTracksAction", is_track_source,
-                        has_selection && is_track_source && track_source.CanRemoveTracks, source
-                    );
-
-                    UpdateAction ("DeleteTracksFromDriveAction", is_track_source,
-                        has_selection && is_track_source && track_source.CanDeleteTracks, source
-                    );
-
+                    UpdateAction ("RemoveTracksAction", is_track_source && track_source.CanRemoveTracks, has_selection, source);
+                    UpdateAction ("DeleteTracksFromDriveAction", is_track_source && track_source.CanDeleteTracks, has_selection, source);
                     UpdateAction ("RemoveTracksFromLibraryAction", source.Parent is LibrarySource, has_selection, null);
                     
                     UpdateAction ("TrackPropertiesAction", in_database, has_selection, null);
@@ -267,6 +265,19 @@ namespace Banshee.Gui
             ResetRating ();
 
             Gtk.Menu menu = action_service.UIManager.GetWidget ("/TrackContextMenu") as Menu;
+            if (menu == null || menu.Children.Length == 0) {
+                return;
+            }
+
+            int visible_children = 0;
+            foreach (Widget child in menu)
+                if (child.Visible)
+                    visible_children++;
+
+            if (visible_children == 0) {
+                return;
+            }
+
             menu.Show (); 
             menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
         }
@@ -283,11 +294,14 @@ namespace Banshee.Gui
         // Generates the menu of playlists to which you can add the selected tracks.
         private void OnAddToPlaylistMenu (object o, EventArgs args)
         {
-            Gdk.Pixbuf pl_pb = Gdk.Pixbuf.LoadFromResource ("source-playlist-16.png");
             Source active_source = ServiceManager.SourceManager.ActiveSource;
 
             // TODO find just the menu that was activated instead of modifying all proxies
-            foreach (MenuItem menu in (o as Gtk.Action).Proxies) {
+            foreach (Widget proxy_widget in (o as Gtk.Action).Proxies) {
+                MenuItem menu = proxy_widget as MenuItem;
+                if (menu == null)
+                    continue;
+
                 Menu submenu = new Menu ();
                 menu.Submenu = submenu;
 
@@ -303,7 +317,7 @@ namespace Banshee.Gui
                         }
                         
                         PlaylistMenuItem item = new PlaylistMenuItem (playlist);
-                        item.Image = new Gtk.Image (pl_pb);
+                        item.Image = new Gtk.Image ("playlist-source", IconSize.Menu);
                         item.Activated += OnAddToExistingPlaylist;
                         item.Sensitive = playlist != active_source;
                         submenu.Append (item);
@@ -321,14 +335,14 @@ namespace Banshee.Gui
             playlist.Save ();
             playlist.PrimarySource.AddChildSource (playlist);
             ThreadAssist.SpawnFromMain (delegate {
-                playlist.AddSelectedTracks (TrackSelector.TrackModel);
+                playlist.AddSelectedTracks (ActiveSource);
             });
         }
 
         private void OnAddToExistingPlaylist (object o, EventArgs args)
         {
             ThreadAssist.SpawnFromMain (delegate {
-                ((PlaylistMenuItem)o).Playlist.AddSelectedTracks (TrackSelector.TrackModel);
+                ((PlaylistMenuItem)o).Playlist.AddSelectedTracks (ActiveSource);
             });
         }
 
@@ -358,7 +372,7 @@ namespace Banshee.Gui
                     }
 
                     ThreadAssist.SpawnFromMain (delegate {
-                        library.RemoveSelectedTracks (source.TrackModel as TrackListDatabaseModel);
+                        library.RemoveSelectedTracks (source.TrackModel as DatabaseTrackListModel);
                     });
                 }
             }
