@@ -54,8 +54,7 @@ namespace Banshee.Lastfm.Radio
     {
         private static string generic_name = Catalog.GetString ("Last.fm Station");
         
-        private LastfmTrackListModel track_model;
-        private SourceMessage status_message;
+        private MemoryTrackListModel track_model;
         
         private LastfmSource lastfm;
         public LastfmSource LastfmSource {
@@ -93,6 +92,10 @@ namespace Banshee.Lastfm.Radio
 
         private int dbid;
         
+        protected override string TypeUniqueId {
+            get { return Convert.ToString (dbid); }
+        }
+        
         // For StationSources that already exist in the db
         protected StationSource (LastfmSource lastfm, int dbId, string name, string type, string arg, int playCount) : base (generic_name, name, 150)
         {
@@ -103,7 +106,7 @@ namespace Banshee.Lastfm.Radio
             PlayCount = playCount;
             Station = Type.GetStationFor (arg);
 
-            Initialize ();
+            StationInitialize ();
         }
 
         public StationSource (LastfmSource lastfm, string name, string type, string arg) : base (generic_name, name, 150)
@@ -115,14 +118,14 @@ namespace Banshee.Lastfm.Radio
 
             Save ();
 
-            Initialize ();
+            StationInitialize ();
         }
 
-        private void Initialize ()
+        private void StationInitialize ()
         {
-            track_model = new LastfmTrackListModel ();
+            track_model = new MemoryTrackListModel ();
 
-            ServiceManager.PlayerEngine.StateChanged += OnPlayerStateChanged;
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, PlayerEvent.StateChange);
             lastfm.Connection.StateChanged += HandleConnectionStateChanged;
 
             Properties.SetString ("GtkActionPath", "/LastfmStationSourcePopup");
@@ -134,7 +137,7 @@ namespace Banshee.Lastfm.Radio
 
         public void Dispose ()
         {
-            ServiceManager.PlayerEngine.StateChanged -= OnPlayerStateChanged;
+            ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
             lastfm.Connection.StateChanged -= HandleConnectionStateChanged;
         }
 
@@ -178,9 +181,6 @@ namespace Banshee.Lastfm.Radio
         public override void Activate ()
         {
             base.Activate ();
-            //shuffle = (action_service.GlobalActions ["ShuffleAction"] as ToggleAction).Active;
-            //(action_service.GlobalActions ["ShuffleAction"] as ToggleAction).Active = false;
-            //Globals.ActionManager["ShuffleAction"].Sensitive = false;
 
             //action_service.GlobalActions ["PreviousAction"].Sensitive = false;
 
@@ -206,9 +206,6 @@ namespace Banshee.Lastfm.Radio
 
         public override void Deactivate ()
         {
-            //(Globals.ActionManager["ShuffleAction"] as ToggleAction).Active = shuffle;
-            //Globals.ActionManager["ShuffleAction"].Sensitive = true;
-
             //Globals.ActionManager["PreviousAction"].Sensitive = true;
         }
 
@@ -235,43 +232,16 @@ namespace Banshee.Lastfm.Radio
             }
         }
 
-        private void SetStatus (string message, bool error)
+        public override void SetStatus (string message, bool error)
         {
-            SetStatus (message, error, ConnectionState.Connected);
+            base.SetStatus (message, error);
+            LastfmSource.SetStatus (status_message, lastfm, error, ConnectionState.Connected);
         }
 
-        private void SetStatus (string message, bool error, ConnectionState state)
+        public void SetStatus (string message, bool error, ConnectionState state)
         {
-            if (status_message == null) {
-                status_message = new SourceMessage (this);
-                PushMessage (status_message);
-            }
-            
-            string status_name = String.Format ("<i>{0}</i>", GLib.Markup.EscapeText (Name));
-            
-            status_message.FreezeNotify ();
-            status_message.Text = String.Format (GLib.Markup.EscapeText (message), status_name);
-            status_message.CanClose = !error;
-            status_message.IsSpinning = !error;
-            status_message.SetIconName (error ? "dialog-error" : null);
-            status_message.ClearActions ();
-            
-            if (error && (state == ConnectionState.NoAccount || state == ConnectionState.InvalidAccount)) {
-                status_message.AddAction (new MessageAction (Catalog.GetString ("Account Settings"),
-                    delegate { lastfm.Actions.ShowLoginDialog (); }));
-                status_message.AddAction (new MessageAction (Catalog.GetString ("Join Last.fm"),
-                    delegate { lastfm.Account.SignUp (); }));
-            }
-            
-            status_message.ThawNotify ();
-        }
-
-        private void HideStatus ()
-        {
-            if (status_message != null) {
-                RemoveMessage (status_message);
-                status_message = null;
-            }
+            base.SetStatus (message, error);
+            LastfmSource.SetStatus (status_message, lastfm, error, state);
         }
 
         /*public override void ShowPropertiesDialog ()
@@ -369,9 +339,10 @@ namespace Banshee.Lastfm.Radio
             });
         }
 
-        private void OnPlayerStateChanged (object o, PlayerEngineStateArgs args)
+        private void OnPlayerEvent (PlayerEventArgs args)
         {
-            if (args.State == PlayerEngineState.Loaded && track_model.Contains (ServiceManager.PlayerEngine.CurrentTrack)) {
+            if (((PlayerEventStateChangeArgs)args).Current == PlayerState.Loaded && 
+                track_model.Contains (ServiceManager.PlayerEngine.CurrentTrack)) {
                 CurrentTrack = ServiceManager.PlayerEngine.CurrentTrack;
 
                 lock (track_model) {
@@ -426,6 +397,11 @@ namespace Banshee.Lastfm.Radio
             );
         }
 
+        public override bool CanRename {
+            get { return true; }
+        }
+
+
 #region ITrackModelSource Implementation
 
         public TrackListModel TrackModel {
@@ -454,6 +430,10 @@ namespace Banshee.Lastfm.Radio
             throw new Exception ("Should not call DeleteSelectedTracks on StationSource");
         }
 
+        public bool CanAddTracks {
+            get { return false; }
+        }
+
         public bool CanRemoveTracks {
             get { return false; }
         }
@@ -502,14 +482,6 @@ namespace Banshee.Lastfm.Radio
             Save ();
         }
         
-        public override bool CanSearch {
-            get { return false; }
-        }
-
-        public override int Count {
-            get { return 0; }
-        }
-
         public override bool HasProperties {
             get { return true; }
         }
