@@ -35,9 +35,8 @@ using Selection = Hyena.Collections.Selection;
 
 namespace Hyena.Data.Gui
 {
-    public partial class ListView<T> : Container
+    public partial class ListView<T> : Widget
     {
-        private int last_click_row_index = -1;
         private int focused_row_index = -1;
 
         private Adjustment vadjustment;
@@ -45,6 +44,11 @@ namespace Hyena.Data.Gui
             get { return vadjustment; }
         }
         
+        private Adjustment hadjustment;
+        public Adjustment Hadjustment {
+            get { return hadjustment; }
+        }
+        
         private SelectionProxy selection_proxy = new SelectionProxy ();
         public SelectionProxy SelectionProxy {
             get { return selection_proxy; }
@@ -198,7 +202,7 @@ namespace Hyena.Data.Gui
         protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
         {
             HasFocus = true;
-            if (header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
+            if (header_visible && header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
                 return OnHeaderButtonPressEvent (evnt);
             } else if (list_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) && model != null) {
                 return OnListButtonPressEvent (evnt);
@@ -233,6 +237,7 @@ namespace Hyena.Data.Gui
                     pressed_column_index = column_c.Index;
                     pressed_column_x_start = x;
                     pressed_column_x_offset = pressed_column_x_start - column_c.X1;
+                    pressed_column_x_start_hadjustment = (int)hadjustment.Value;
                 }
             }
             
@@ -261,10 +266,8 @@ namespace Hyena.Data.Gui
                 return true;
             }
 
-            if (evnt.Button == 1 && evnt.Type == Gdk.EventType.TwoButtonPress && 
-                row_index == last_click_row_index) {
+            if (evnt.Button == 1 && evnt.Type == Gdk.EventType.TwoButtonPress) {
                 OnRowActivated ();
-                last_click_row_index = -1;
             } else {
                 if ((evnt.State & Gdk.ModifierType.ControlMask) != 0) {
                     if (evnt.Button == 3) {
@@ -291,10 +294,7 @@ namespace Hyena.Data.Gui
                 FocusRow (row_index);
 
                 if (evnt.Button == 3) {
-                    last_click_row_index = -1;
                     OnPopupMenu ();
-                } else {
-                    last_click_row_index = row_index;
                 }
             }
             
@@ -309,6 +309,7 @@ namespace Hyena.Data.Gui
         protected override bool OnButtonReleaseEvent (Gdk.EventButton evnt)
         {
             OnDragSourceSet ();
+            StopDragScroll ();
             
             if (resizing_column_index >= 0) {
                 pressed_column_index = -1;
@@ -326,7 +327,7 @@ namespace Hyena.Data.Gui
                 return true;
             }
             
-            if (header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
+            if (header_visible && header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
                 return OnHeaderButtonRelease (evnt);
             } else if (list_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) && model != null &&
                 (evnt.State & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0) {
@@ -338,16 +339,21 @@ namespace Hyena.Data.Gui
         
         private bool OnHeaderButtonRelease (Gdk.EventButton evnt)
         {
-            Column column = column_cache[pressed_column_index].Column;
-            if (column != null && Model is ISortable && column is ISortableColumn) {
-                ((ISortable)Model).Sort ((ISortableColumn)column);
-                Model.Reload ();
-                RegenerateColumnCache ();
-                InvalidateHeader ();
+            if (pressed_column_index > 0 && pressed_column_index < column_cache.Length) {
+                Column column = column_cache[pressed_column_index].Column;
+                if (column != null && Model is ISortable && column is ISortableColumn) {
+                    ((ISortable)Model).Sort ((ISortableColumn)column);
+                    Model.Reload ();
+                    RecalculateColumnSizes ();
+                    RegenerateColumnCache ();
+                    InvalidateHeader ();
+                }
+                
+                pressed_column_index = -1;
+                return true;
+            } else {
+                return false;
             }
-            
-            pressed_column_index = -1;
-            return true;
         }
         
         private bool OnListButtonRelease (Gdk.EventButton evnt)
@@ -388,36 +394,9 @@ namespace Hyena.Data.Gui
                 InvalidateList ();
             }
             
-            if (pressed_column_is_dragging) {
-                GdkWindow.Cursor = drag_cursor;
-                
-                Column swap_column = GetColumnAt (x);
-                
-                if (swap_column != null) {
-                    CachedColumn swap_column_c = GetCachedColumnForColumn (swap_column);
-                    bool reorder = false;
-                    
-                    if (swap_column_c.Index < pressed_column_index) {
-                        // Moving from right to left
-                        reorder = pressed_column_x_drag <= swap_column_c.X1 + swap_column_c.Width / 2;
-                    } else if (swap_column_c.Index > pressed_column_index) {
-                        // Moving from left to right
-                        reorder = pressed_column_x_drag + column_cache[pressed_column_index].Width >= 
-                            swap_column_c.X1 + swap_column_c.Width / 2;
-                    }
-                    
-                    if (reorder) {
-                        int actual_pressed_index = ColumnController.IndexOf (column_cache[pressed_column_index].Column);
-                        int actual_swap_index = ColumnController.IndexOf (swap_column_c.Column);
-                        ColumnController.Reorder (actual_pressed_index, actual_swap_index);
-                        pressed_column_index = swap_column_c.Index;
-                        RegenerateColumnCache ();
-                    }
-                }
-                
-                pressed_column_x_drag = x - pressed_column_x_offset;
-                
-                QueueDraw ();
+            pressed_column_x = x;
+            
+            if (OnMotionNotifyEvent (x)) {
                 return true;
             }
             
@@ -427,12 +406,59 @@ namespace Hyena.Data.Gui
                 : null;
             
             if (resizing_column_index >= 0) {
-                ResizeColumn ((double)x);
+                ResizeColumn (x);
             }
             
             return true;
         }
         
+        private bool OnMotionNotifyEvent (int x)
+        {
+            if (!pressed_column_is_dragging) {
+                return false;
+            }
+            
+            OnDragScroll (OnDragHScrollTimeout, header_interaction_alloc.Width * 0.1, header_interaction_alloc.Width, x);
+            
+            GdkWindow.Cursor = drag_cursor;
+            
+            Column swap_column = GetColumnAt (x);
+            
+            if (swap_column != null) {
+                CachedColumn swap_column_c = GetCachedColumnForColumn (swap_column);
+                bool reorder = false;
+                
+                if (swap_column_c.Index < pressed_column_index) {
+                    // Moving from right to left
+                    reorder = pressed_column_x_drag <= swap_column_c.X1 + swap_column_c.Width / 2;
+                } else if (swap_column_c.Index > pressed_column_index) {
+                    // Moving from left to right
+                    reorder = pressed_column_x_drag + column_cache[pressed_column_index].Width >= 
+                        swap_column_c.X1 + swap_column_c.Width / 2;
+                }
+                
+                if (reorder) {
+                    int actual_pressed_index = ColumnController.IndexOf (column_cache[pressed_column_index].Column);
+                    int actual_swap_index = ColumnController.IndexOf (swap_column_c.Column);
+                    ColumnController.Reorder (actual_pressed_index, actual_swap_index);
+                    pressed_column_index = swap_column_c.Index;
+                    RegenerateColumnCache ();
+                }
+            }
+            
+            pressed_column_x_drag = x - pressed_column_x_offset - (pressed_column_x_start_hadjustment - (int)hadjustment.Value);
+            
+            QueueDraw ();
+            return true;
+        }
+        
+        private bool OnDragHScrollTimeout ()
+        {
+            ScrollTo (hadjustment, hadjustment.Value + (drag_scroll_velocity * drag_scroll_velocity_max));
+            OnMotionNotifyEvent (pressed_column_x);
+            return true;
+        }
+        
         protected override bool OnLeaveNotifyEvent (Gdk.EventCrossing evnt)
         {
             GdkWindow.Cursor = null;
@@ -484,43 +510,80 @@ namespace Hyena.Data.Gui
 
 #region Adjustments & Scrolling
         
+        private void UpdateAdjustments ()
+        {
+            UpdateAdjustments (null, null);
+        }
+        
         private void UpdateAdjustments (Adjustment hadj, Adjustment vadj)
         {
+            if (hadj != null) {
+                hadjustment = hadj;
+            }
+            
             if (vadj != null) {
                 vadjustment = vadj;
             }
             
+            if (hadjustment != null) {
+                hadjustment.Upper = header_width;
+                hadjustment.StepIncrement = 10.0;
+                if (hadjustment.Value + hadjustment.PageSize > hadjustment.Upper) {
+                    hadjustment.Value = hadjustment.Upper - hadjustment.PageSize;
+                }
+            }
+            
             if (vadjustment != null && model != null) {
                 vadjustment.Upper = (RowHeight * (model.Count));
                 vadjustment.StepIncrement = RowHeight;
+                if (vadjustment.Value + vadjustment.PageSize > vadjustment.Upper) {
+                    vadjustment.Value = vadjustment.Upper - vadjustment.PageSize;
+                }
             }
             
+            hadjustment.Change ();
             vadjustment.Change ();
         }
         
-        private void OnAdjustmentChanged (object o, EventArgs args)
+        private void OnHadjustmentChanged (object o, EventArgs args)
+        {
+            InvalidateHeader ();
+            InvalidateList (false);
+        }
+        
+        private void OnVadjustmentChanged (object o, EventArgs args)
         {
             InvalidateList (false);
         }
         
         public void ScrollTo (double val)
         {
-            vadjustment.Value = Math.Max (0.0, Math.Min (val, vadjustment.Upper - vadjustment.PageSize));
+            ScrollTo (vadjustment, val);
+        }
+        
+        private void ScrollTo (Adjustment adjustment, double val)
+        {
+            adjustment.Value = Math.Max (0.0, Math.Min (val, adjustment.Upper - adjustment.PageSize));
         }
 
-        public void ScrollToRow (int row_index)
+        public void ScrollTo (int index)
         {
-            ScrollTo (GetYAtRow (row_index));
+            ScrollTo (GetYAtRow (index));
+        }
+
+        public void CenterOn (int index)
+        {
+            ScrollTo (index - RowsInView/2 + 1);
         }
                 
         protected override void OnSetScrollAdjustments (Adjustment hadj, Adjustment vadj)
         {
-            if (vadj == null || hadj == null) {
+            if (hadj == null || vadj == null) {
                 return;
             }
             
-            vadj.ValueChanged += OnAdjustmentChanged;
-            hadj.ValueChanged += OnAdjustmentChanged;
+            hadj.ValueChanged += OnHadjustmentChanged;
+            vadj.ValueChanged += OnVadjustmentChanged;
             
             UpdateAdjustments (hadj, vadj);
         }
