@@ -32,7 +32,9 @@ using System.Data;
 
 namespace Hyena.Data.Sqlite
 {
-    public class SqliteModelCache<T> : DictionaryModelCache<T> where T : ICacheableItem
+    //public delegate void AggregatesUpdatedEventHandler (IDataReader reader);
+
+    public class SqliteModelCache<T> : DictionaryModelCache<T> where T : ICacheableItem, new ()
     {
         private HyenaSqliteConnection connection;
         private ICacheableDatabaseModel model;
@@ -45,6 +47,7 @@ namespace Hyena.Data.Sqlite
         private HyenaSqliteCommand save_selection_command;
         private HyenaSqliteCommand get_selection_command;
 
+        private string select_str;
         private string reload_sql;
         private long uid;
         private long selection_uid;
@@ -52,8 +55,7 @@ namespace Hyena.Data.Sqlite
         // private bool warm;
         private long first_order_id;
 
-        public delegate void AggregatesUpdatedEventHandler (IDataReader reader);
-        public event AggregatesUpdatedEventHandler AggregatesUpdated;
+        public event Action<IDataReader> AggregatesUpdated;
 
         public SqliteModelCache (HyenaSqliteConnection connection,
                            string uuid,
@@ -94,26 +96,25 @@ namespace Hyena.Data.Sqlite
             }
 
             uid = FindOrCreateCacheModelId (String.Format ("{0}-{1}", uuid, typeof(T).Name));
-            selection_uid = FindOrCreateCacheModelId (String.Format ("{0}-{1}-Selection", uuid, typeof(T).Name));
+
+            if (model.Selection != null) {
+                selection_uid = FindOrCreateCacheModelId (String.Format ("{0}-{1}-Selection", uuid, typeof(T).Name));
+            }
 
             if (model.CachesJoinTableEntries) {
-                select_range_command = new HyenaSqliteCommand (
-                    String.Format (@"
-                        SELECT {0}, {5}.ItemID  FROM {1}
-                            INNER JOIN {2}
-                                ON {3} = {2}.{4}
-                            INNER JOIN {5}
-                                ON {2}.{6} = {5}.ItemID
-                            WHERE
-                                {5}.ModelID = {7} {8}
-                                {9}
-                            LIMIT ?, ?",
-                        provider.Select, provider.From,
-                        model.JoinTable, provider.PrimaryKey, model.JoinColumn,
-                        CacheTableName, model.JoinPrimaryKey, uid,
-                        String.IsNullOrEmpty (provider.Where) ? null : "AND",
-                        provider.Where
-                    )
+                select_str = String.Format (
+                    @"SELECT {0}, {5}.ItemID  FROM {1}
+                        INNER JOIN {2}
+                            ON {3} = {2}.{4}
+                        INNER JOIN {5}
+                            ON {2}.{6} = {5}.ItemID
+                        WHERE
+                            {5}.ModelID = {7} {8} {9}",
+                    provider.Select, provider.From,
+                    model.JoinTable, provider.PrimaryKey, model.JoinColumn,
+                    CacheTableName, model.JoinPrimaryKey, uid,
+                    String.IsNullOrEmpty (provider.Where) ? null : "AND",
+                    provider.Where
                 );
 
                 select_single_command = new HyenaSqliteCommand (
@@ -132,22 +133,18 @@ namespace Hyena.Data.Sqlite
                     CacheTableName, uid, model.JoinPrimaryKey
                 );
             } else {
-                select_range_command = new HyenaSqliteCommand (
-                    String.Format (@"
-                        SELECT {0}, {2}.ItemID FROM {1}
-                            INNER JOIN {2} 
-                                ON {3} = {2}.ItemID
-                            WHERE
-                                {2}.ModelID = {4} {5}
-                                {6}
-                            LIMIT ?, ?",
-                        provider.Select, provider.From, CacheTableName,
-                        provider.PrimaryKey, uid,
-                        String.IsNullOrEmpty (provider.Where) ? String.Empty : "AND",
-                        provider.Where
-                    )
+                select_str = String.Format (
+                    @"SELECT {0}, {2}.ItemID FROM {1}
+                        INNER JOIN {2} 
+                            ON {3} = {2}.ItemID
+                        WHERE
+                            {2}.ModelID = {4} {5} {6}",
+                    provider.Select, provider.From, CacheTableName,
+                    provider.PrimaryKey, uid,
+                    String.IsNullOrEmpty (provider.Where) ? String.Empty : "AND",
+                    provider.Where
                 );
-            
+
                 select_single_command = new HyenaSqliteCommand (
                     String.Format (@"
                         SELECT OrderID FROM {0}
@@ -164,7 +161,11 @@ namespace Hyena.Data.Sqlite
                     CacheTableName, uid, provider.PrimaryKey
                 );
             }
-            
+
+            select_range_command = new HyenaSqliteCommand (
+                String.Format ("{0} {1}", select_str, "LIMIT ?, ?")
+            );
+
             select_first_command = new HyenaSqliteCommand (
                 String.Format (
                     "SELECT OrderID FROM {0} WHERE ModelID = {1} LIMIT 1",
@@ -172,19 +173,21 @@ namespace Hyena.Data.Sqlite
                 )
             );
 
-            delete_selection_command = new HyenaSqliteCommand (String.Format (
-                "DELETE FROM {0} WHERE ModelID = {1}", CacheTableName, selection_uid
-            ));
-            
-            save_selection_command = new HyenaSqliteCommand (String.Format (
-                "INSERT INTO {0} (ModelID, ItemID) SELECT {1}, ItemID FROM {0} WHERE ModelID = {2} LIMIT ?, ?",
-                CacheTableName, selection_uid, uid
-            ));
+            if (model.Selection != null) {
+                delete_selection_command = new HyenaSqliteCommand (String.Format (
+                    "DELETE FROM {0} WHERE ModelID = {1}", CacheTableName, selection_uid
+                ));
+                
+                save_selection_command = new HyenaSqliteCommand (String.Format (
+                    "INSERT INTO {0} (ModelID, ItemID) SELECT {1}, ItemID FROM {0} WHERE ModelID = {2} LIMIT ?, ?",
+                    CacheTableName, selection_uid, uid
+                ));
 
-            get_selection_command = new HyenaSqliteCommand (String.Format (
-                "SELECT OrderID FROM {0} WHERE ModelID = {1} AND ItemID IN (SELECT ItemID FROM {0} WHERE ModelID = {2})",
-                CacheTableName, uid, selection_uid
-            ));
+                get_selection_command = new HyenaSqliteCommand (String.Format (
+                    "SELECT OrderID FROM {0} WHERE ModelID = {1} AND ItemID IN (SELECT ItemID FROM {0} WHERE ModelID = {2})",
+                    CacheTableName, uid, selection_uid
+                ));
+            }
         }
 
         private bool has_select_all_item = false;
@@ -239,20 +242,50 @@ namespace Hyena.Data.Sqlite
             }
         }
 
+        private HyenaSqliteCommand get_single_command;
+        private string last_get_single_fragment;
+        public T GetSingle (string fragment, params object [] args)
+        {
+            if (fragment != last_get_single_fragment || get_single_command == null) {
+                last_get_single_fragment = fragment;
+                get_single_command = new HyenaSqliteCommand (
+                    String.Format ("{0} {1} {2}", select_str, fragment, "LIMIT 1")
+                );
+            }
+
+            using (IDataReader reader = connection.Query (get_single_command, args)) {
+                if (reader.Read ()) {
+                    T item = provider.Load (reader);
+                    item.CacheEntryId = Convert.ToInt64 (reader[reader.FieldCount - 1]);
+                    item.CacheModelId = uid;
+                    return item;
+                }
+            }
+            return default(T);
+        }
+
+
         private HyenaSqliteCommand last_reload_command;
         private string last_reload_fragment;
 
         public override void Reload ()
         {
             lock (this) {
-                if (last_reload_fragment != model.ReloadFragment) {
+                if (last_reload_fragment != model.ReloadFragment || last_reload_command == null) {
                     last_reload_fragment = model.ReloadFragment;
-                    last_reload_command = new HyenaSqliteCommand (reload_sql + last_reload_fragment);
+                    last_reload_command = new HyenaSqliteCommand (String.Format ("{0}{1}", reload_sql, last_reload_fragment));
                 }
 
                 Clear ();
                 //Log.DebugFormat ("Reloading {0} with {1}", model, last_reload_command.Text);
                 connection.Execute (last_reload_command);
+            }
+        }
+
+        public override void Clear ()
+        {
+            lock (this) {
+                base.Clear ();
                 first_order_id = -1;
             }
         }
@@ -260,7 +293,9 @@ namespace Hyena.Data.Sqlite
         private bool saved_selection = false;
         public void SaveSelection ()
         {
-            if (model.Selection.Count > 0 && !(has_select_all_item && model.Selection.AllSelected)) {
+            if (model.Selection != null && model.Selection.Count > 0 &&
+                !(has_select_all_item && model.Selection.AllSelected))
+            {
                 connection.Execute (delete_selection_command);
                 saved_selection = true;
 
@@ -284,29 +319,29 @@ namespace Hyena.Data.Sqlite
 
         public void RestoreSelection ()
         {
-            long selected_id = -1;
-            long first_id = FirstOrderId;
-
-            // Compensate for the first, fake 'All *' item
-            if (has_select_all_item) {
-                first_id -= 1;
-            }
-
-            model.Selection.Clear (false);
-
             if (saved_selection) {
+                long selected_id = -1;
+                long first_id = FirstOrderId;
+
+                // Compensate for the first, fake 'All *' item
+                if (has_select_all_item) {
+                    first_id -= 1;
+                }
+
+                model.Selection.Clear (false);
+
                 using (IDataReader reader = connection.Query (get_selection_command)) {
                     while (reader.Read ()) {
                         selected_id = Convert.ToInt64 (reader[0]) - first_id;
                         model.Selection.QuietSelect ((int)selected_id);
                     }
                 }
-            }
 
-            if (has_select_all_item && model.Selection.Count == 0) {
-                model.Selection.QuietSelect (0);
+                if (has_select_all_item && model.Selection.Count == 0) {
+                    model.Selection.QuietSelect (0);
+                }
+                saved_selection = false;
             }
-            saved_selection = false;
         }
 
         protected override void FetchSet (long offset, long limit)
@@ -316,7 +351,7 @@ namespace Hyena.Data.Sqlite
                     T item;
                     while (reader.Read ()) {
                         if (!ContainsKey (offset)) {
-                            item = provider.Load (reader, (int) offset);
+                            item = provider.Load (reader);
                             item.CacheEntryId = Convert.ToInt64 (reader[reader.FieldCount - 1]);
                             item.CacheModelId = uid;
                             Add (offset, item);
@@ -333,7 +368,7 @@ namespace Hyena.Data.Sqlite
                 if (reader.Read ()) {
                     rows = Convert.ToInt64 (reader[0]);
 
-                    AggregatesUpdatedEventHandler handler = AggregatesUpdated;
+                    Action<IDataReader> handler = AggregatesUpdated;
                     if (handler != null) {
                         handler (reader);
                     }
