@@ -34,9 +34,10 @@ using System.Text;
 
 namespace Hyena.Data.Sqlite
 {
-    public abstract class SqliteModelProvider<T>
+    public class SqliteModelProvider<T> where T : new ()
     {
         private readonly List<DatabaseColumn> columns = new List<DatabaseColumn> ();
+        private readonly List<DatabaseColumn> select_columns = new List<DatabaseColumn> ();
         private readonly List<VirtualDatabaseColumn> virtual_columns = new List<VirtualDatabaseColumn> ();
         
         private DatabaseColumn key;
@@ -45,10 +46,12 @@ namespace Hyena.Data.Sqlite
         private HyenaSqliteCommand create_command;
         private HyenaSqliteCommand insert_command;
         private HyenaSqliteCommand update_command;
+        private HyenaSqliteCommand delete_command;
         private HyenaSqliteCommand select_command;
         private HyenaSqliteCommand select_range_command;
         private HyenaSqliteCommand select_single_command;
         
+        private string table_name;
         private string primary_key;
         private string select;
         private string from;
@@ -56,12 +59,24 @@ namespace Hyena.Data.Sqlite
         
         private const string HYENA_DATABASE_NAME = "hyena_database_master";
 
-        public abstract string TableName { get; }
-        protected abstract int ModelVersion { get; }
-        protected abstract int DatabaseVersion { get; }
-        protected abstract void MigrateTable (int old_version);
-        protected abstract void MigrateDatabase (int old_version);
-        protected abstract T MakeNewObject (int offset);
+        public virtual string TableName { get { return table_name; } }
+
+        protected virtual int ModelVersion { get { return 1; } }
+
+        protected virtual int DatabaseVersion { get { return 1; } }
+
+        protected virtual void MigrateTable (int old_version)
+        {
+        }
+
+        protected virtual void MigrateDatabase (int old_version)
+        {
+        }
+
+        protected virtual T MakeNewObject ()
+        {
+            return new T ();
+        }
         
         protected virtual string HyenaTableName {
             get { return "HyenaModelVersions"; }
@@ -71,9 +86,20 @@ namespace Hyena.Data.Sqlite
         {
             this.connection = connection;
         }
+        
+        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name) : this (connection)
+        {
+            this.table_name = table_name;
+            Init ();
+        }
 
         protected void Init ()
         {
+            foreach (FieldInfo field in typeof(T).GetFields (BindingFlags.Instance | BindingFlags.Public)) {
+                foreach (Attribute attribute in field.GetCustomAttributes (true)) {
+                    AddColumn (field, attribute);
+                }
+            }
             foreach (FieldInfo field in typeof(T).GetFields (BindingFlags.Instance | BindingFlags.NonPublic)) {
                 foreach (Attribute attribute in field.GetCustomAttributes (true)) {
                     AddColumn (field, attribute);
@@ -89,7 +115,6 @@ namespace Hyena.Data.Sqlite
                     AddColumn (property, attribute);
                 }
             }
-            
             if (key == null) {
                 throw new Exception (String.Format ("The {0} table does not have a primary key", TableName));
             }
@@ -198,6 +223,10 @@ namespace Hyena.Data.Sqlite
                 }
                 
                 columns.Add (c);
+
+                if (column.Select) {
+                    select_columns.Add (c);
+                }
                 
                 if ((c.Constraints & DatabaseColumnConstraints.PrimaryKey) > 0) {
                     if (key != null) {
@@ -240,45 +269,48 @@ namespace Hyena.Data.Sqlite
             }
         }
         
-        protected virtual void PrepareInsertCommand (T target)
+        protected virtual object [] GetInsertParams (T target)
         {
-            object [] values = new object [columns.Count];
+            // TODO create an instance variable object array and reuse it? beware threading issues
+            object [] values = new object [columns.Count - 1];
+            int j = 0;
             for (int i = 0; i < columns.Count; i++) {
                 if (columns[i] != key) {
-                    values[i] = columns[i].GetValue (target);
-                } else {
-                    // On insert, the key needs to be NULL to be automatically set by Sqlite
-                    values[i] = null;
+                    values[j] = columns[i].GetValue (target);
+                    j++;
                 }
             }
-            InsertCommand.ApplyValues (values);
+            return values;
         }
         
         protected int Insert (T target)
         {
-            PrepareInsertCommand (target);
-            return connection.Execute (InsertCommand);
+            return connection.Execute (InsertCommand, GetInsertParams (target));
         }
 
-        protected virtual void PrepareUpdateCommand (T target)
+        protected object [] GetUpdateParams (T target)
         {
-            object [] values = new object [columns.Count + 1];
+            // TODO create an instance variable object array and reuse it? beware threading issues
+            object [] values = new object [columns.Count];
+            int j = 0;
             for (int i = 0; i < columns.Count; i++) {
-                values [i] = columns[i].GetValue (target);
+                if (columns[i] != key) {
+                    values[j] = columns[i].GetValue (target);
+                    j++;
+                }
             }
-            values[columns.Count] = key.GetValue (target);
-            UpdateCommand.ApplyValues (values);
+            values[j] = key.GetValue (target);
+            return values;
         }
         
         protected void Update (T target)
         {
-            PrepareUpdateCommand (target);
-            connection.Execute (UpdateCommand);
+            connection.Execute (UpdateCommand, GetUpdateParams (target));
         }
         
-        public T Load (IDataReader reader, int index)
+        public T Load (IDataReader reader)
         {
-            T item = MakeNewObject (index);
+            T item = MakeNewObject ();
             Load (reader, item);
             return item;
         }
@@ -289,7 +321,7 @@ namespace Hyena.Data.Sqlite
             
             AbstractDatabaseColumn bad_column = null;
             try {
-                foreach (DatabaseColumn column in columns) {
+                foreach (DatabaseColumn column in select_columns) {
                     bad_column = column;
                     column.SetValue (target, reader, i++);
                 }
@@ -306,52 +338,116 @@ namespace Hyena.Data.Sqlite
             }
         }
         
-        protected virtual void PrepareSelectCommand ()
-        {
-        }
-        
         public IEnumerable<T> FetchAll ()
         {
-            PrepareSelectCommand ();
-            int i = 1;
             using (IDataReader reader = connection.Query (SelectCommand)) {
                 while (reader.Read ()) {
-                    yield return Load (reader, i++);
+                    yield return Load (reader);
                 }
             }
         }
-        
-        protected virtual void PrepareSelectRangeCommand (int offset, int limit)
+
+        public T FetchFirstMatching (string condition)
         {
-            SelectRangeCommand.ApplyValues (offset, limit);
+            HyenaSqliteCommand fetch_matching_command = new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+            using (IDataReader reader = connection.Query (fetch_matching_command)) {
+                if (reader.Read ()) {
+                    return Load (reader);
+                }
+            }
+            return default(T);
+        }
+        
+        public IEnumerable<T> FetchAllMatching (string condition)
+        {
+            HyenaSqliteCommand fetch_matching_command = new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+            using (IDataReader reader = connection.Query (fetch_matching_command)) {
+                while (reader.Read ()) {
+                    yield return Load (reader);
+                }
+            }
         }
         
         public IEnumerable<T> FetchRange (int offset, int limit)
         {
-            PrepareSelectRangeCommand (offset, limit);
-            using (IDataReader reader = connection.Query (SelectRangeCommand)) {
+            using (IDataReader reader = connection.Query (SelectRangeCommand, offset, limit)) {
                 while (reader.Read ()) {
-                    yield return Load (reader, offset++);
+                    yield return Load (reader);
                 }
             }
         }
         
-        protected virtual void PrepareSelectSingleCommand (object id)
-        {
-            SelectSingleCommand.ApplyValues (id);
-        }
-        
         public T FetchSingle (int id)
         {
-            PrepareSelectSingleCommand (id);
-            using (IDataReader reader = connection.Query (SelectSingleCommand)) {
+            return FetchSingle ((long) id);
+        }
+        
+        public T FetchSingle (long id)
+        {
+            using (IDataReader reader = connection.Query (SelectSingleCommand, id)) {
                 if (reader.Read ()) {
-                    return Load (reader, id);
+                    return Load (reader);
                 }
             }
             return default(T);
         }
         
+        protected long PrimaryKeyFor (T item)
+        {
+            return (long) key.GetValue (item);
+        }
+        
+        public void Delete (long id)
+        {
+            connection.Execute (delete_command, id);
+        }
+        
+        public void Delete (T item)
+        {
+            Delete (PrimaryKeyFor (item));
+        }
+        
+        public void Delete (IEnumerable<T> items)
+        {
+            List<long> ids = new List<long> ();
+            long id;
+            foreach (T item in items) {
+                id = PrimaryKeyFor (item);
+                if (id > 0)
+                    ids.Add (id);
+            }
+            
+            if (ids.Count > 0)
+                connection.Execute (delete_command, ids.ToArray ());
+        }
+
+        public bool Refresh (T item)
+        {
+            if (key == null || item == null)
+                return false;
+
+            int id = (int) key.GetValue (item);
+            if (id < 1)
+                return false;
+
+            using (IDataReader reader = connection.Query (SelectSingleCommand, id)) {
+                if (reader.Read ()) {
+                    Load (reader);
+                    return true;
+                }
+            }
+            return false;
+        }
+
+        public void Copy (T original, T copy)
+        {
+            foreach (DatabaseColumn column in select_columns) {
+                if (column != key) {
+                    column.SetValue (copy, column.GetRawValue (original));
+                }
+            }
+        }
+        
         protected virtual HyenaSqliteCommand CreateCommand {
             get {
                 if (create_command == null) {
@@ -383,14 +479,16 @@ namespace Hyena.Data.Sqlite
                     StringBuilder vals = new StringBuilder ();
                     bool first = true;
                     foreach (DatabaseColumn column in columns) {
-                        if (first) {
-                            first = false;
-                        } else {
-                            cols.Append (',');
-                            vals.Append (',');
+                        if (column != key) {
+                            if (first) {
+                                first = false;
+                            } else {
+                                cols.Append (',');
+                                vals.Append (',');
+                            }
+                            cols.Append (column.Name);
+                            vals.Append ('?');
                         }
-                        cols.Append (column.Name);
-                        vals.Append ('?');
                     }
 
                     insert_command = new HyenaSqliteCommand (String.Format (
@@ -411,13 +509,15 @@ namespace Hyena.Data.Sqlite
                     builder.Append (" SET ");
                     bool first = true;
                     foreach (DatabaseColumn column in columns) {
-                        if (first) {
-                            first = false;
-                        } else {
-                            builder.Append (',');
+                        if (column != key) {
+                            if (first) {
+                                first = false;
+                            } else {
+                                builder.Append (',');
+                            }
+                            builder.Append (column.Name);
+                            builder.Append (" = ?");
                         }
-                        builder.Append (column.Name);
-                        builder.Append (" = ?");
                     }
                     builder.Append (" WHERE ");
                     builder.Append (key.Name);
@@ -433,10 +533,8 @@ namespace Hyena.Data.Sqlite
                 if (select_command == null) {
                     select_command = new HyenaSqliteCommand (
                         String.Format (
-                            "SELECT {0} FROM {1}{2}{3}",
-                            Select, From,
-                            (String.IsNullOrEmpty (Where) ? String.Empty : " WHERE "),
-                            Where
+                            "SELECT {0} FROM {1} WHERE {2}",
+                            Select, From, String.IsNullOrEmpty (Where) ? "1=1" : Where
                         )
                     );
                 }
@@ -476,6 +574,17 @@ namespace Hyena.Data.Sqlite
             }
         }
         
+        protected virtual HyenaSqliteCommand DeleteCommand {
+            get {
+                if (delete_command == null) {
+                    delete_command = new HyenaSqliteCommand (String.Format (
+                        "DELETE FROM {0} WHERE {1} IN (?)", TableName, PrimaryKey
+                    ));
+                }
+                return delete_command;
+            }
+        }
+        
         public virtual string Select {
             get {
                 if (select == null) {
@@ -516,7 +625,7 @@ namespace Hyena.Data.Sqlite
         {
             StringBuilder select_builder = new StringBuilder ();
             bool first = true;
-            foreach (DatabaseColumn column in columns) {
+            foreach (DatabaseColumn column in select_columns) {
                 if (first) {
                     first = false;
                 } else {
